#
# Copyright (c) 1996 Regents of the University of California.
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#       This product includes software developed by the MASH Research
#       Group at the University of California Berkeley.
# 4. Neither the name of the University nor of the Research Group may be
#    used to endorse or promote products derived from this software without
#    specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# @(#) $Header: /usr/src/mash/repository/mash/mash-1/tcl/applications/camera-server/camera-srvr.tcl,v 1.6 1998/08/21 20:03:54 hodes Exp $
#


import SerialChannel/Camera-Sony-EVID30 SerialChannel/Camera-Canon-VCC1 \
		UDPServer 
		#RoomManager/Client

# Camera messaging interface -- handles all transmission of state
# announcements of the underlying agent. <p>
#
Class UDPServer/Camera -superclass UDPServer

# only uses the port num and performs multiple unicasts to
# all known clients <P> XXX need to time them out; enable mcast
#
UDPServer/Camera public init {addrspec parent {mtu 1500}} {
    $self instvar parent_ pending_
    set parent_ $parent
    set pending_ -1

    eval [list $self] next $addrspec $mtu
}

# Receive a msg to the Camera Server.
# <br>
# format of received announcement is:
#  "method args ..."
#
UDPServer/Camera private recv {addr port data size} {
    #puts "Msg: $addr\[$size\]: $data"

    $self instvar parent_ pending_
    set c [$parent_ set cam_]

    set theMethod [lindex $data 0]
    set theArgs [lrange $data 1 end]

    if {[lsearch [[$c info class] info instprocs] $theMethod] != -1} {
	# disallow list:
	if {$theMethod == "init"} {return}
	# check number of args
	if {[llength $theArgs] != \
		[llength [[$c info class] info instargs $theMethod]]} {
	    puts "Bad arg count to $theMethod"
	    return
	}
	# okay, invoke the camera command
	#puts -nonewline "invoking: $c $theMethod $theArgs... "
	set reply [eval $c $theMethod $theArgs]
	# don't return reply; should be subset of full state update

	if {$pending_ != -1} {
	    after cancel $pending_
	}
	# wait a bit to coalesce bursts of updates
	set pending_ [after 300 $self announce_state]
    } else {
	if {$theMethod == "update_state"} {
	    $self announce_state
	} else {
	    puts "msg with method '$theMethod' not found.  ignored."
	}
    }
}

#
# a wrapper for sending a complete state update
#
UDPServer/Camera private announce_state {} {
    $self instvar parent_ pending_

    set c [$parent_ set cam_]
    set tmp "z: [$c set zoomPerc_]"
    append tmp " t: [$c set tiltPerc_]"
    append tmp " p: [$c set panPerc_]"
    append tmp " pre: "
    foreach i "[$c array names presets_]" {
	append tmp "$i [$c get_preset $i] "
    }

    $self announce "$tmp"
    set pending_ -1
}

# a UI-less server for controlling local camera(s) 
#<br>
# simple wrapper for accepting remote commands (method invocations) 
# and passing them on the the underlying camera object
#
Class CameraServer

#
# init takes 3 args: addrspec, type of camera (the class has to be 
# pre-imported or locally importable), and the device name
#
CameraServer public init {addrspec {type SerialChannel/Camera-Sony-EVID30} \
		{device /dev/cuac00}} {

	$self instvar cam_ al_

	import $type
	set al_ [new UDPServer/Camera $addrspec $self]
	set cam_ [new $type]
	# hack until -device $dev works in constructor
	$cam_ close
	$cam_ device $device			
	$cam_ open
}



