# This file contains the Video agent used by many psvp mechanisms.

import VideoAgent

Class PsvpVideoAgent -superclass VideoAgent

PsvpVideoAgent instproc init {spec} {
    eval $self next $self $spec;
    set local_src [$self local];
    if {$local_src != ""} {
	$local_src handler [new Module/VideoDecoder/Null];
    }
}

PsvpVideoAgent instproc get_source_by_id srcid {
    $self instvar sources_;
    
    if {$srcid == "*"} {
	foreach s $sources_ {
	    if {$s != [$self local]} {
		set d [$s handler];
		if {$d != ""} {
		    if {[$d info class] == "Module/VideoDecoder/Null"} {
			return $s;
		    }
		}
	    }
	}
    } else {	
	foreach s $sources_ {
	    if {$srcid == [$s srcid]} {
		return $s;
	    }
	}
    }
    return "";
}

PsvpVideoAgent instproc delete_decoder decoder {
    $self instvar decoders_;
    
    set idx [lsearch $decoders_ $decoder];
    if {$idx == -1} {
	return;
    }
    
    set src [$decoder set src_];
    
    lreplace $decoders_ $idx $idx "";
    
    set null_decoder [$self create_decoder $src];
    
    $src handler $null_decoder;
    lappend decoders_ $null_decoder;
}

PsvpVideoAgent instproc set_create_decoder_callback {srcid cmd} {
    $self instvar callbacks_;
    
    set callbacks_($srcid) $cmd;
}

PsvpVideoAgent instproc set_src_decoder {src decoder} {
    $self instvar decoders_ sources_;
    
    if {[lsearch $sources_ $src] == -1} {
	return;
    }
    
    $decoder set agent_ $self;
    $decoder set src_ $src;
    
    set old_decoder [$src handler];
    
    if {$old_decoder != ""} {
	delete $old_decoder;
	set idx [lsearch $decoders_ $old_decoder];
	if {$idx != -1} {
	    set decoders_ [lreplace $decoders_ $idx $idx];
	}
    }

    $src handler $decoder;

    lappend decoders_ $decoder;

}

PsvpVideoAgent instproc create_decoder src {
    $self instvar callbacks_;
    set d [$src handler];
    if {$d != ""} {
	delete $d
    }

    set id [$src srcid];

    if {![info exists callbacks_($id)]} {
	if {[info exists callbacks_(*)]} {
	    set decoder [eval $callbacks_(*) $src];
	    unset callbacks_(*);
	} else {
	    set decoder [new Module/VideoDecoder/Null];
	}
    } else {
	set decoder [eval $callbacks_($id) $src];
	unset callbacks_($id);
    }
    
    $decoder set agent_ $self;
    $decoder set src_ $src;

    return $decoder;
}
